@@ -19,7 +19,7 @@ use Carp;
 use 5.008;
 # major.minor.bugfix, the latter two with 3 digits each
 # or major.minor_alpha
-our $VERSION = '3.001000';
+our $VERSION = '3.002000';
 $VERSION = eval $VERSION;
 our %features = qw(array 1 hash 1);
 
@@ -299,6 +299,9 @@ Additional fun with negative values, optionally followed by comma-separated list
 
 	$self->find_config_files();
 
+	# Chain of config files being parsed, to be able to check for inclusion loops.
+	$self->{parse_chain} = [];
+
 	return $self;
 }
 
@@ -319,7 +322,19 @@ sub define
 	{
 		$self->{type}{$pd->{long}}  = $type;
 		$self->INT_verb_msg("defining $pd->{long} / $pd->{short} of type $typename[$type]\n");
-		$self->{param}{$pd->{long}} = $pd->{value};
+		# If the definition value is a reference, make a deep copy of it
+		# instead of copying the reference. This keeps the definition
+		# and default value unchanged, for reproducible multiple runs of
+		# the parser.
+		if(ref $pd->{value})
+		{
+			require Storable; # Only require it if there is really the need.
+			$self->{param}{$pd->{long}} = Storable::dclone($pd->{value});
+		}
+		else
+		{
+			$self->{param}{$pd->{long}} = $pd->{value};
+		}
 		$self->{long}{$pd->{short}} = $pd->{long} if $pd->{short} ne '';
 		$self->{short}{$pd->{long}} = $pd->{short};
 		$self->{help}{$pd->{long}}  = ref $pd->{help} ? $pd->{help} : \$pd->{help};
@@ -1042,8 +1057,13 @@ sub parse_file
 	{
 		$self->INT_error("Couldn't find config file $confname!") unless $self->{config}{nocomplain};
 	}
+	elsif(grep {$_ eq $file} @{$self->{parse_chain}})
+	{
+		$self->INT_error("Trying to parse config file $file twice in one chain!");
+	}
 	elsif($cdat->open($file, '<'))
 	{
+		push(@{$self->{parse_chain}}, $file);
 		push(@{$self->{files}}, $file);
 		if(defined $self->{config}{binmode})
 		{
@@ -1118,7 +1138,9 @@ sub parse_file
 						{
 							my $dir = dirname($file);
 							$dir = File::Spec->rel2abs($dir);
-							$incfile = File::Spec->catfile($dir, $incfile); 
+							$incfile = File::Spec->catfile($dir, $incfile);
+							$incfile .= '.conf'
+								unless -e $incfile;
 						}
 						$self->parse_file($incfile, $construct);
 					}
@@ -1211,6 +1233,7 @@ sub parse_file
 		}
 		$cdat->close();
 		$self->INT_verb_msg("... done parsing.\n");
+		pop(@{$self->{parse_chain}});
 	}
 	else{ $self->INT_error("Couldn't open config file $file! ($!)") unless $self->{config}{nocomplain}; }
 
@@ -1728,9 +1751,9 @@ And that is actually one reason for not just using another markup or serializati
 
 Comments in the config file start with "#" and are supposed to be on lines on their own (otherwise, they will get parsed as part of a parameter value). Special tokens for the parser (see see parse_file method, with the optional parameter) start with "=", the important one is inclusion of other config files:
 
-	=include anotherconfigfile
+	=include anotherconfigfile[.conf]
 
-This will locate anotherconfigfile (relative paths are relative to the current config file, also search paths are used) and load its settings in-place, then continue with the current file. Note that there is no check for inclusion loops (yet). The file name starts with the first non-whitespace after B<=include> and continues until the end of line. No quoting.
+This will locate anotherconfigfile either via absolute path or relative to the currently parsed file. Search paths are B<not> used (this was documented erroneously before). Its settings are loaded in-place and parsing continues with the current file.  The file name starts with the first non-whitespace after B<=include> and continues until the end of line. No quoting. If the given name does not exist, the ending .conf is added and that one tried instead. The parsing aborts (croaks) if an inclusion loop is detected.
 
 There are more tokens in a fully annotated output of config files. Example:
 
@@ -2120,7 +2143,7 @@ Thomas Orgis <thomas@orgis.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2004-2014, Thomas Orgis.
+Copyright (C) 2004-2016, Thomas Orgis.
 
 This module is free software; you
 can redistribute it and/or modify it under the same terms
